﻿using System;
using System.Diagnostics;
using Windows.ApplicationModel;
using Windows.ApplicationModel.Activation;
using Windows.ApplicationModel.Core;
using Windows.UI.Core;
using Windows.UI.Popups;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace SimpleOoms
{
    sealed partial class App : Application
    {

        #region Init

        public static bool IsActivationDone { get; set; }
        public static bool HandleOomExceptions { get; set; }
        public static bool DoFailFast { get; set; }

        private static string versionString = string.Empty;
        public static string VersionString
        {
            get
            {
                if (string.IsNullOrEmpty(versionString))
                {
                    PackageVersion packageVersion = Package.Current.Id.Version;
                    versionString = string.Format("{0}.{1}.{2}.{3}",
                        packageVersion.Major, packageVersion.Minor, packageVersion.Build, packageVersion.Revision);
                }
                return versionString;
            }
        }

        public App()
        {
			UnhandledException += App_UnhandledException;
			InitializeComponent();
            Suspending += OnSuspending;
        }

        protected override void OnLaunched(LaunchActivatedEventArgs e)
        {
            if (e.PrelaunchActivated)
            {
                return;
            }
            Frame rootFrame = Window.Current.Content as Frame;
            if (rootFrame == null)
            {
                rootFrame = new Frame();
                rootFrame.Language = Windows.Globalization.ApplicationLanguages.Languages[0];
                rootFrame.NavigationFailed += OnNavigationFailed;
                if (e.PreviousExecutionState == ApplicationExecutionState.Terminated)
                {
                }

                Window.Current.Content = rootFrame;
            }

            if (rootFrame.Content == null)
            {
                rootFrame.Navigate(typeof(MainPage), e.Arguments);
            }
            Window.Current.Activate();
        }

        private void OnNavigationFailed(object sender, NavigationFailedEventArgs e)
        {
            throw new Exception("Failed to load Page " + e.SourcePageType.FullName);
        }

        private void OnSuspending(object sender, SuspendingEventArgs e)
        {
            var deferral = e.SuspendingOperation.GetDeferral();
            deferral.Complete();
        }

        #endregion


        // UnhandledException handler will get called for unhandled OOM exceptions, 
        // but not for allocation failures in the XAML framework.
        private void App_UnhandledException(object sender, UnhandledExceptionEventArgs e)
		{
            if (Debugger.IsAttached)
            {
                Debugger.Break();
            }
            else
            {
                if (e.Exception is OutOfMemoryException && !HandleOomExceptions)
                {
                    return;
                }
                if (e.Exception != null && !string.IsNullOrEmpty(e.Exception.Message))
				{
					ShowMessage(e.Exception.Message);
				}
				else
				{
					ShowMessage("Unhandled Exception");
				}
				e.Handled = true;
			}
		}

		internal async static void ShowMessage(string message, bool forceShow = false)
		{
			if (Debugger.IsAttached && !forceShow)
			{
				Debug.WriteLine(message);
			}
			else
			{
				if (IsActivationDone)
				{
					if (CoreApplication.MainView.CoreWindow.Dispatcher.HasThreadAccess)
					{
						MessageDialog dialog = new MessageDialog(message);
						await dialog.ShowAsync();
					}
					else
					{
						await CoreApplication.MainView.CoreWindow.Dispatcher.RunAsync(
							CoreDispatcherPriority.Normal, async () => {
								MessageDialog dialog = new MessageDialog(message);
								await dialog.ShowAsync();
						});
					}
				}
			}
		}

	}
}
