﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using Windows.Storage;
using Windows.Storage.Streams;
using Windows.System;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media.Imaging;
using Windows.UI.Xaml.Navigation;

namespace SimpleOoms
{
    public sealed partial class MainPage : Page
    {

        #region Init

        private const int MB = 1024 * 1024;
        private const ulong allocationSize = 10 * 1024 * 1024;
        private List<byte[]> consumedMemory;

        public MainPage()
        {
            App.HandleOomExceptions = true;
            App.IsActivationDone = true;
            InitializeComponent();
            version.Text = App.VersionString;
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            if (e.NavigationMode == NavigationMode.New)
            {
                consumedMemory = new List<byte[]>();
            }
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            if (e.NavigationMode == NavigationMode.Back)
            {
                consumedMemory.Clear();
                consumedMemory = null;
                GC.Collect();
            }
        }

        #endregion


        private void handleOom_Click(object sender, RoutedEventArgs e)
        {
            App.HandleOomExceptions = (bool)handleOom.IsChecked;
        }

        private void failFast_Click(object sender, RoutedEventArgs e)
        {
            App.DoFailFast = (bool)failFast.IsChecked;
        }

        private void scopedAlloc_Click(object sender, RoutedEventArgs e)
        {
            status.Log("allocating byte array.");
            ShowMemoryUsage();

            if (App.HandleOomExceptions)
            {
                try
                {
                    LoopingMemoryAllocator();
                }
                catch (OutOfMemoryException oomEx)
                {
                    status.Log(oomEx.ToString());
                    if (App.DoFailFast)
                    {
                        Environment.FailFast("OutOfMemoryException in DoSomething");
                    }
                }
                catch (Exception ex)
                {
                    status.Log(ex.Message);
                }
            }
            else
            {
                // This will eventually throw an OutOfMemoryException, which will invoke the 
                // App.App_UnhandledException handler.
                LoopingMemoryAllocator();
            }
        }

        private void LoopingMemoryAllocator()
        {
            while (true)
            {
                ShowMemoryUsage();
                consumedMemory.Add(new byte[allocationSize]);
            }
        }

        async private void unscopedAlloc_Click(object sender, RoutedEventArgs e)
        {
            status.Log("allocating bitmaps.");
            ShowMemoryUsage();

            List<BitmapImage> bitmaps = new List<BitmapImage>();
            StorageFile file = await StorageFile.GetFileFromApplicationUriAsync(
                new Uri("ms-appx:///assets/SplashScreen.scale-100.png"));
            IRandomAccessStream stream = await file.OpenAsync(FileAccessMode.Read);

            while (true)
            {
                try
                {
                    ShowMemoryUsage();
                    stream.Seek(0);
                    BitmapImage bitmap = new BitmapImage();
                    // This will eventually cause an allocation failure, which will not 
                    // result in any handleable exceptions to the app.
                    bitmap.SetSource(stream);
                    bitmaps.Add(bitmap);
                }
                catch (OutOfMemoryException oom)
                {
                    status.Log(oom.ToString());
                    break;
                }
                catch (Exception ex)
                {
                    status.Log(ex.Message);
                    break;
                }
            }
        }

        private void ShowMemoryUsage()
        {
            try
            {
                string usageText = string.Format(
                    CultureInfo.CurrentCulture, "usage={0}MB, limit={1}MB",
                    Math.Ceiling((double)MemoryManager.AppMemoryUsage / MB),
                    Math.Ceiling((double)MemoryManager.AppMemoryUsageLimit / MB));
                status.Log(usageText);
            }
            catch (Exception ex)
            {
                status.Log(ex.Message);
            }
        }


///////////////////////////////////////////////// simplified for documentation.
private void DoSomething() {
    List<byte[]> buffer = new List<byte[]>();
    try {
        while (true)
            buffer.Add(new byte[10 * 1024 * 1024]);
    }
    catch (OutOfMemoryException oomEx) {
        // Handle the exception and recover.
        Debug.WriteLine(oomEx.ToString());
        Environment.FailFast("OutOfMemoryException in DoSomething");
    }
}

private void DoAnother(IRandomAccessStream stream) {
    List<BitmapImage> bitmaps = new List<BitmapImage>();
    while (true) {
        stream.Seek(0);
        BitmapImage bitmap = new BitmapImage();
        bitmap.SetSource(stream);
        bitmaps.Add(bitmap);
    }
}

    }
}
