﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Windows.System;
using Windows.System.Diagnostics;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace TestMemoryApis
{
    public sealed partial class MainPage : Page
    {

        #region Click handlers

        public MainPage()
        {
            InitializeComponent();
        }

        private void tryMemoryIntensiveOperation_Click(object sender, RoutedEventArgs e)
        {
            TryMemoryIntensiveOperation();
        }

        private void moreMemoryIntensiveOperation_Click(object sender, RoutedEventArgs e)
        {
            TryMemoryIntensiveOperation2();
        }

        private void getUsageLevel_Click(object sender, RoutedEventArgs e)
        {
            GetUsageLevel();
        }

        private void registerMemoryCallbacks_Click(object sender, RoutedEventArgs e)
        {
            RegisterMemoryManagerCallbacks();
        }

        private void getDiagnostics_Click(object sender, RoutedEventArgs e)
        {
            GetDiagnostics();
        }

        private void allocate_Click(object sender, RoutedEventArgs e)
        {
            AllocateMemory();
        }

        private void consume_Click(object sender, RoutedEventArgs e)
        {
            ConsumeMemory();
        }

        private void release_Click(object sender, RoutedEventArgs e)
        {
            ReleaseMemory();
        }

        private void memoryStatus_Click(object sender, RoutedEventArgs e)
        {
            GetMemoryStatus();
        }

        private void pageFile_Click(object sender, RoutedEventArgs e)
        {
            GetPageFileSize();
        }

        #endregion


        private const ulong SOME_APP_DEFINED_VALUE = 10 * 1024 * 1024;


        #region MemoryManager

        private void TryMemoryIntensiveOperation(){
            ulong limit = MemoryManager.AppMemoryUsageLimit;
            ulong usage = MemoryManager.AppMemoryUsage;
            ulong headroom = limit - usage;
            if (headroom > SOME_APP_DEFINED_VALUE){
                // Do memory-intensive operation.
            }
        }

        private void TryMemoryIntensiveOperation2(){
            AppMemoryReport r = MemoryManager.GetAppMemoryReport();
            ulong headroom = r.TotalCommitLimit - r.TotalCommitUsage;
            if (headroom > SOME_APP_DEFINED_VALUE){
                // Do memory-intensive operation.
            }
        }

        private void GetUsageLevel() { 
            AppMemoryUsageLevel usageLevel = MemoryManager.AppMemoryUsageLevel;
            switch (usageLevel)
            {
                //case AppMemoryUsageLevel.OverLimit:
                    // Take urgent action to reduce memory usage.
                //    break;
                case AppMemoryUsageLevel.High:
                    // Take action to reduce memory usage.
                    break;
                case AppMemoryUsageLevel.Medium:
                    // Continue to monitor memory usage.
                    break;
                case AppMemoryUsageLevel.Low:
                    // Can start memory-intensive operations.
                    break;
            }
        }

        #endregion


        #region MemoryManager Events

        public void RegisterMemoryManagerCallbacks(){
            MemoryManager.AppMemoryUsageIncreased += OnUsageIncreased;
            MemoryManager.AppMemoryUsageDecreased += OnUsageDecreased;
        }

        private void OnUsageIncreased(object sender, object e){
            AppMemoryUsageLevel level = MemoryManager.AppMemoryUsageLevel;
            if (level == AppMemoryUsageLevel.High) { 
                //|| level == AppMemoryUsageLevel.OverLimit){
                // Take action to reduce memory usage.
            }
            else if (level == AppMemoryUsageLevel.Medium){
                // Continue to monitor memory usage.
            }
        }

        private void OnUsageDecreased(object sender, object e){
            AppMemoryUsageLevel level = MemoryManager.AppMemoryUsageLevel;
            if (level == AppMemoryUsageLevel.Medium){
                // Back down at an acceptable level.
            }
            else if (level == AppMemoryUsageLevel.Low){
                // Can start memory-intensive operations.
            }
        }

        private void _RegisterMemoryManagerCallbacks(){
            MemoryManager.AppMemoryUsageLimitChanging += OnMemoryLimitChanging;
        }

        private void OnMemoryLimitChanging(
            object sender, AppMemoryUsageLimitChangingEventArgs e){
            if (MemoryManager.AppMemoryUsage >= e.NewLimit){
                // Take urgent action to reduce memory usage as much as possible.
            }
            // Alternatve implementation
            //if (MemoryManager.AppMemoryUsageLevel == 
            //    AppMemoryUsageLevel.OverLimit){
            //    //Take urgent action to reduce memory usage as much as possible.
            //}
        }

        #endregion


        #region Diagnostics

        private void Trace(string s, ulong u)
        {
            Debug.WriteLine(s, (double)u / 1024 / 1024);
        }

        private void GetDiagnostics(){
            Trace("commit={0:N}", MemoryManager.AppMemoryUsage);
            ProcessMemoryReport r1 = MemoryManager.GetProcessMemoryReport();
            Trace("private w/s={0:N}", r1.PrivateWorkingSetUsage);
            Trace("total w/s={0:N}", r1.TotalWorkingSetUsage);

            ProcessDiagnosticInfo info = 
                ProcessDiagnosticInfo.GetForCurrentProcess();
            ProcessMemoryUsageReport r2 = info.MemoryUsage.GetReport();
            Trace("peak w/s={0:N}", r2.PeakWorkingSetSizeInBytes);
        }

        private List<byte[]> buffer = new List<byte[]>();

        private void AllocateMemory(){
            byte[] data = new byte[SOME_APP_DEFINED_VALUE];
            buffer.Add(data);
        }

        private void ConsumeMemory(){
            byte[] data = new byte[SOME_APP_DEFINED_VALUE];
            for (int i = 0; i < data.Length; i++)
                data[i] = 1;
            buffer.Add(data);
        }

        private void ReleaseMemory(){
            if (buffer.Count > 0)
                buffer.RemoveAt(0);
            GC.Collect();
        }

        #endregion


        #region MemoryStatus

        [StructLayout(LayoutKind.Sequential)]
        private class MEMORYSTATUSEX
        {
            public uint dwLength;
            public uint dwMemoryLoad;
            public ulong ullTotalPhys;
            public ulong ullAvailPhys;
            public ulong ullTotalPageFile;
            public ulong ullAvailPageFile;
            public ulong ullTotalVirtual;
            public ulong ullAvailVirtual;
            public ulong ullAvailExtendedVirtual;
            public MEMORYSTATUSEX()
            {
                dwLength = (uint)Marshal.SizeOf<MEMORYSTATUSEX>();
            }
        }

        [DllImport("kernelbase.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        private static extern bool GlobalMemoryStatusEx(
            [In, Out] MEMORYSTATUSEX lpBuffer);

        private void GetMemoryStatus()
        {
            MEMORYSTATUSEX status = new MEMORYSTATUSEX();
            if (GlobalMemoryStatusEx(status))
            {
                Trace("TotalPhys={0:N}", status.ullTotalPhys);
                Trace("AvailPhys={0:N}", status.ullAvailPhys);
                Trace("TotalPageFile={0:N}", status.ullTotalPageFile);
                Trace("AvailPageFile={0:N}", status.ullAvailPageFile);
                // ...etc.
            }
        }

        private void GetPageFileSize()
        {
            MEMORYSTATUSEX status = new MEMORYSTATUSEX();
            if (GlobalMemoryStatusEx(status))
            {
                ulong pageFile = 
                    status.ullTotalPageFile - status.ullTotalPhys;
                Trace("PageFile={0:N}", pageFile);
            }
        }

        #endregion

    }
}
